/*
** Module   : DynamicTempTableImpl.java
** Abstract : Declaration of some TEMP-TABLES dynamic features 
*              (attributes, methods).
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -----------------------Description------------------------
** 002 OM  20130129 Created initial version.
*/

package com.goldencode.p2j.persist;

import com.goldencode.p2j.util.*;

/**
 * Declaration of dynamic TEMP-TABLES attributes and methods.
 * This interface is needed by {@link CommonHandle} in order to store 
 * the handle to a temp table.
 * 
 * @author om
 */
public interface DynamicTempTable
extends Buffer
{
   /**
    * Duplicates the structure of an existing table, copying the field 
    * definitions, and optionally, the indexes.
    * This method is the P2J equivalent of <code>CREATE-LIKE</code> method
    * of Progress 4GL.
    *
    * @param   hbuf
    *          A handler assigned to a table buffer to create the copy.
    *
    * @return  TBA: Doc are not explicit, most likely true on success
    */
   public logical createTableLike(handle hbuf);

   /**
    * Adds a new field to the temp-table. This overloaded form creates field  
    * with properties se to their default values.
    * This method is the P2J equivalent of <code>ADD-NEW-FIELD</code> method
    * of Progress 4GL.
    *
    * @param   name
    *          The name of the field to be created in the temp-table.
    * @param   type
    *          The data type of the specified field.
    *
    * @return  TBA: Doc are not explicit, most likely true on success
    */
   public logical addNewField(String name, String type);

   /**
    * Adds a field with the specified properties to the temp-table.
    * This method is the P2J equivalent of <code>ADD-NEW-FIELD</code> method
    * of Progress 4GL.
    * 
    * @param   name
    *          The name of the field to be created in the temp-table.
    * @param   type
    *          The data type of the specified field.
    * @param   extent
    *          An integer expression specifying the extent of an array.
    * @param   format
    *          The data format for the defined data type. If null or unknown, 
    *          the default format will be used.
    * @param   initial
    *          An expression that evaluates to the initial value of the 
    *          defined field. 
    *          TODO: this method will probably be overloaded because of this.
    * @param   label
    *          The label of the defined field. If null or unknown the name 
    *          parameter will be used.
    * @param   columnLabel
    *          The label of the column associated with the defined field
    *          
    * @return  TBA: Doc are not explicit, most likely true on success
    */
   public logical addNewField(String name,
                              String type,
                              int    extent,
                              String format,
                              String initial,
                              String label,
                              String columnLabel);

   /**
    * Adds a field with the specified properties to the temp-table.
    * This method is the P2J equivalent of <code>ADD-NEW-FIELD</code> method
    * of Progress 4GL.
    * 
    * @param   name
    *          The name of the field to be created in the temp-table.
    * @param   type
    *          The data type of the specified field.
    * @param   extent
    *          An integer expression specifying the extent of an array.
    * @param   format
    *          The data format for the defined data type. If null or unknown, 
    *          the default format will be used.
    * @param   initial
    *          An expression that evaluates to the initial value of the 
    *          defined field. 
    *          TODO: this method will probably be overloaded because of this.
    * @param   label
    *          The label of the defined field. If null or unknown the name 
    *          parameter will be used.
    * @param   columnLabel
    *          The label of the column associated with the defined field
    *          
    * @return  TBA: Doc are not explicit, most likely true on success
    */
   public logical addNewField(character    name,
                              character    type,
                              int          extent,
                              character    format,
                              BaseDataType initial,
                              character    label,
                              character    columnLabel);

   /**
    * Adds a new index with the specified name to the temp-table. Newly 
    * created idex is empty and its components must be added individually 
    * with the <code>ADD-INDEX-FIELD()</code> method.
    * This form of the overloaded method creates an index that is not unique, 
    * not primary and not a word index.
    * This method is the P2J equivalent of <code>ADD-NEW-INDEX</code> method
    * of Progress 4GL.
    * 
    * @param   name
    *          The name of the index to be created.
    * 
    * @return  TBA: Doc are not explicit, most likely true on success
    */
   public logical addNewIndex(String name);

   /**
    * Adds a new index with the specified name to the temp-table. Newly 
    * created idex is empty and its components must be added individually 
    * with the <code>ADD-INDEX-FIELD()</code> method.
    * This method is the P2J equivalent of <code>ADD-NEW-INDEX</code> method
    * of Progress 4GL.
    *
    * @param   name
    *          The name of the index to be created.
    * @param   unique
    *          TRUE if this index is unique.
    * @param   primary
    *          TRUE if this is the primary index.
    * @param   word
    *          TRUE if this is a word index.
    * 
    * @return  TBA: Doc are not explicit, most likely true on success
    */
   public logical addNewIndex(String  name,
                              boolean unique,
                              boolean primary,
                              boolean word);

   /**
    * Adds a new empty index with the specified name to the temp-table. Index 
    * components must be added with the <code>ADD-INDEX-FIELD()</code> method.
    * This method is the P2J equivalent of <code>ADD-NEW-INDEX</code> method
    * of Progress 4GL.
    *
    * @param   name
    *          The name of the index to be created.
    * @param   unique
    *          TRUE if this index is unique.
    * @param   primary
    *          TRUE if this is the primary index.
    * @param   word
    *          TRUE if this is a word index.
    * 
    * @return  TBA: Doc are not explicit, most likely true on success.
    */
   public logical addNewIndex(String  name,
                              logical unique,
                              logical primary,
                              logical word);

   /**
    * Adds to the temp-table a field like the one specified.
    * This method is the P2J equivalent of <code>ADD-LIKE-FIELD</code> method
    * of Progress 4GL.
    * 
    * @param   name
    *          The name of the field to be created in the temp-table.
    * @param   source
    *          Database field name from which to copy the field. The table 
    *          name must be qualified with the database name.
    * 
    * @return  TBA: Doc are not explicit, most likely true on success
    */
   public logical addFieldLike(String name, String source);

   /**
    * Adds to the temp-table a field like the one specified.
    * This method is the P2J equivalent of <code>ADD-LIKE-FIELD</code> method
    * of Progress 4GL.
    * 
    * @param   name
    *          The name of the field to be created in the temp-table.
    * @param   source
    *          Database field name from which to copy the field. The table 
    *          name must be qualified with the database name.
    * 
    * @return  TBA: Doc are not explicit, most likely true on success
    */
   public logical addFieldLike(character name, character source);

   /**
    * Adds to the temp-table a field like the one specified.
    * This method is the P2J equivalent of <code>ADD-LIKE-FIELD</code> method
    * of Progress 4GL.
    * 
    * @param   name
    *          The name of the field to be created in the temp-table.
    * @param   source
    *          Handle to field name from which to copy the field.
    * 
    * @return  TBA: Doc are not explicit, most likely true on success
    */
   public logical addFieldLike(character name, handle source);

   /**
    * Adds to the temp-table an index like the one specified.
    * This method is the P2J equivalent of <code>ADD-LIKE-FIELD</code> method
    * of Progress 4GL.
    * 
    * @param   name
    *          The name of the index to which the source index is being copied.
    * @param   sourceName
    *          The name of the index in the source table that is being copied 
    *          to the temp-table.
    * @param   dbTable
    *          The database table name from which to copy the index.
    * 
    * @return  TBA: Doc are not explicit, most likely true on success
    */
   public logical addIndexLike(String name,
                               String sourceName,
                               String dbTable);

   /**
    * Adds to the temp-table an index like the one specified.
    * This method is the P2J equivalent of <code>ADD-LIKE-FIELD</code> method
    * of Progress 4GL.
    * 
    * @param   name
    *          The name of the index to which the source index is being copied.
    * @param   sourceName
    *          The name of the index in the source table that is being copied 
    *          to the temp-table.
    * @param   dbTable
    *          The database table name from which to copy the index.
    * 
    * @return  TBA: Doc are not explicit, most likely true on success
    */
   public logical addIndexLike(character name,
                               character sourceName,
                               character dbTable);

   /**
    * Signals that all the field and index definitions for a temp-table 
    * have been provided. After the call to this method no fields and indexes
    * can be added to this temporary table.
    * This method is the P2J equivalent of <code>TEMP-TABLE-PREPARE</code> 
    * method of Progress 4GL.
    * 
    * @param   name
    *          Temp-table name to be used in subsequent query statements that
    *          refer to this temp-table.
    *
    * @return  TBA: Doc are not explicit, most likely true on success.
    */
   public logical tempTablePrepare(String name);

   /**
    * Signals that all the field and index definitions for a temp-table 
    * have been provided. After the call to this method no fields and indexes
    * can be added to this temporary table.
    * This method is the P2J equivalent of <code>TEMP-TABLE-PREPARE</code> 
    * method of Progress 4GL.
    * 
    * @param   name
    *          Temp-table name to be used in subsequent query statements that
    *          refer to this temp-table.
    *          
    * @return  TBA: Doc are not explicit, most likely true on success.
    */
   public logical tempTablePrepare(character name);

   /**
    * Obtain the handle to the buffer that was created by default for this 
    * temporary table. Every dynamic temp-table is created with at 
    * least one buffer. This buffer’s object handle is returned by this 
    * method. It cannot be called until the table has been prepared using 
    * <code>tempTablePrepare()</code> method.
    * This method is the P2J equivalent of <code>DEFAULT-BUFFER-HANDLE</code>
    * attribute.
    * 
    * @param   object (handle tth)
    * 
    * @return  The default buffer handle for the temporary table.
    */
   public handle getDefaultBufferHandle();

   /**
    * Obtain the handle to a particular field in the buffer.
    * This method is the P2J equivalent of <code>BUFFER-FIELD</code> method.
    * 
    * @param   name
    *          The field name.
    *
    * @return  A handle of the buffer of the requested field
    */
   public handle getField(String name);

   /**
    * Obtain the handle to a particular field in the buffer.
    * This method is the P2J equivalent of <code>BUFFER-FIELD</code> method.
    * 
    * @param   name
    *          The field name.
    *
    * @return  A handle of the buffer of the requested field
    */
   public handle getField(character name);

   /**
    * Obtain the handle to a particular field in the buffer.
    * This method is the P2J equivalent of <code>BUFFER-FIELD</code> method.
    * 
    * @param   fieldNumber
    *          The number of a fields in the buffer.
    *
    * @return  A handle of the buffer of the requested field
    */
   public handle getField(int fieldNumber);

   /**
    * Obtain the handle to a particular field in the buffer.
    * This method is the P2J equivalent of <code>BUFFER-FIELD</code> method.
    * 
    * @param   name
    *          The number of a fields in the buffer.
    *
    * @return  A handle of the buffer of the requested field
    */
   public handle getField(NumberType fieldNo);
}
